package tools.cache2;

import java.lang.management.ManagementFactory;
import java.util.Comparator;

import org.netkernel.layer0.nkf.*;
import org.netkernel.layer0.nkf.impl.NKFEndpointImpl;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.representation.IHDSPredicate;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IResponse;
import org.netkernel.request.IResponseMeta;

import com.ten60.netkernel.cache.se.representation2.ConcurrentCache;

public class ViewCache extends StandardAccessorImpl
{
	public ViewCache()
	{	declareThreadSafe();
	}
	
	public void onSource(INKFRequestContext context) throws Exception
	{
		ConcurrentCache cache=ConcurrentCache.getInstance();
		if (cache==null)
		{	throw new NKFException("ConcurrentCache not initialised");
		}
		IHDSNode param = (IHDSNode)context.source("httpRequest:/params",IHDSNode.class);
		if (param.getChildren().length==0)
		{	HDSBuilder b=new HDSBuilder();
			b.addNode("match","");
			b.addNode("max","20");
			b.addNode("sortby","costRev");
			param=b.getRoot();
		}

		String viewHash = (String)param.getFirstValue("view");
		if (viewHash!=null)
		{ 
			int viewHashInt = Integer.parseInt(viewHash);
			IResponse resp = cache.getByHash(viewHashInt);
			Object rep;
			
			if (resp!=null)
			{
				String mime=(String)resp.getMeta().getUserMetaData().getValue(IResponseMeta.KEY_MIMETYPE);
				if (resp.getRepresentation()!=null && !(resp.getRepresentation() instanceof IBinaryStreamRepresentation))
				{	try
					{	INKFRequest req=context.createRequest("var:dummy");
						req.setVerb(INKFRequestReadOnly.VERB_TRANSREPRESENT);
						req.addPrimaryArgument(resp.getRepresentation());
						req.setRepresentationClass(IBinaryStreamRepresentation.class);
						rep=context.issueRequest(req);
					}
					catch (NKFException e)
					{	rep="Transreption not possible; toString()=\""+resp.getRepresentation().toString()+"\"";
						mime="text/plain";
					}
				}
				else
				{	rep=resp.getRepresentation();
				}

				if (rep!=null )
				{	if (mime==null)
					{	mime="text/plain";
					}
					INKFResponse resp2=context.createResponseFrom(rep);
					resp2.setMimeType(mime);
					resp2.setExpiry(INKFResponse.EXPIRY_ALWAYS);
				}
			}
			else
			{	throw new Exception("resource not found");
			}	
		}
		else
		{	if ("clear".equals(param.getFirstValue("action2")))
			{	cache.clear();
				ManagementFactory.getMemoryMXBean().gc();
				Thread.sleep(100);
				ManagementFactory.getMemoryMXBean().gc();
				Thread.sleep(100);
			}
		
		    IHDSNode cacheState = cache.getState();
		    
		    final String filter=(String)param.getFirstValue("match");
		    IHDSPredicate f1=new IHDSPredicate()
			{	
				public boolean matches(IHDSNode aNode)
				{	return ((String)aNode.getFirstValue("id")).contains(filter);
				}
			};
			
			String sb=(String)param.getFirstValue("sortby");
			final String sortby;
			final boolean rev;
			if (sb.endsWith("Rev"))
			{	rev=true;
				sortby=sb.substring(0,sb.length()-3);
			}
			else
			{	rev=false;
				sortby=sb;
			}
			Comparator<IHDSNode> c = new Comparator<IHDSNode>()
			{
				public int compare(IHDSNode o1, IHDSNode o2)
				{	
					Comparable id1=(Comparable)o1.getFirstValue(sortby);
					Comparable id2=(Comparable)o2.getFirstValue(sortby);
					int v=id1.compareTo(id2);
					if (rev) v=-v;
					return v;
				}
			};
			
		    final int max=Integer.parseInt((String)param.getFirstValue("max"));
		
		
		    IHDSPredicate f2=new IHDSPredicate()
			{	
				int i=0;
				public boolean matches(IHDSNode aNode)
				{	return (i++)<max;
				}
			};
		
			//long t0=System.currentTimeMillis();
			IHDSNodeList items = cacheState.getNodes("/cache/item");
			int size=items.size();
			items = items.filter(f1);
			//long t1=System.currentTimeMillis();
			int itemCount=items.size();
			items = items.sort(c);
			//long t2=System.currentTimeMillis();
			items = items.filter(f2);
			//long t3=System.currentTimeMillis();
			//System.out.println((t1-t0)+" "+(t2-t1)+" "+(t3-t2));
			
			HDSBuilder b=new HDSBuilder();
			b.addNode("hitPercent",cacheState.getFirstValue("/cache/hitPercent"));
			b.addNode("size",size);
			b.addNode("status",cacheState.getFirstValue("/cache/status"));
			b.addNode("count",itemCount);
			
			INKFResponse r = context.createResponseFrom(items);
			r.setExpiry(INKFResponse.EXPIRY_ALWAYS);
			
			INKFRequest req = context.createRequest("active:xslt");
			req.addArgument("operator","res:/tools/cache2/styleCache.xsl");
			req.addArgumentFromResponse("operand",r);
			req.addArgumentByValue("status",b.getRoot());
			req.addArgumentByValue("param",param);
			
			INKFResponseReadOnly subResp=context.issueRequestForResponse(req);
			INKFResponse resp=context.createResponseFrom(subResp);
			resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
			resp.setHeader("WrappedControlPanel", Boolean.TRUE);
		}
	}
	
}
