package tools.classtrace;

import java.net.URL;

import org.netkernel.layer0.boot.IModule;
import org.netkernel.layer0.boot.ModuleManager;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.urii.SimpleIdentifierImpl;
import org.netkernel.layer0.util.SpaceClassLoader;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.urii.IIdentifier;
import org.netkernel.urii.ISpace;
import org.netkernel.util.Utils;

public class ClassTraceAccessor extends StandardAccessorImpl
{
	private IHDSNode mNoData;
	
	public ClassTraceAccessor()
	{	HDSBuilder b=new HDSBuilder();
		b.pushNode("resolve");
		mNoData=b.getRoot();
	}
	
	public void onSource(INKFRequestContext context) throws Exception
	{
		IHDSNode params=context.source("httpRequest:/params",IHDSNode.class);
		String moduleString=(String)params.getFirstValue("module");
		
		IHDSNode data;
	    if (moduleString!=null)
	    {	
	    	IModule module=null;
	    	for (IModule m: ModuleManager.getSingleton().getModules())
	    	{	if (m.getMeta().getIdentifier().equals(moduleString))
	    		{	module=m;
	    			break;
	    		}
	    	}
	        if (module==null)
	        {	throw new Exception("Module not found");
	        }
	    	ClassLoader moduleCL=module.getSpaces()[0].getClassLoader();	    	
	        
	        String className=params.getFirstValue("class").toString();
	        String resourceName=className.replace('.','/')+".class";
	
	        HDSBuilder b = new HDSBuilder();
	        b.pushNode("resolve");
	        
	        ClassLoader cl;
	        try
	        {	cl=moduleCL.loadClass(className).getClassLoader();
	        	b.addNode("classloader", cl!=null?cl.toString():"bootstrap class loader [JDK/JRE]");
	        }
	        catch (Throwable e)
	        {   b.addNode("classloaderError", Utils.throwableToString(e));
	        	cl=null;
	        }
	        
	        if (cl==null)
	        {   cl=moduleCL;
	        }
	        URL url=cl.getResource(resourceName);
	        if (url!=null)
	        {	b.addNode("url",url.toString());
	        }
	        SpaceClassLoader scl=(SpaceClassLoader)moduleCL;
	        scl.appendHDS(b);
	        data=b.getRoot();
	        
	    }
	    else
	    {	data=mNoData;
	    }
	    
	    INKFRequest req=context.createRequest("active:xslt");
	    req.addArgument("operator","res:/tools/classtrace/styleTrace.xsl");
	    req.addArgument("operand","active:moduleListDoc");
	    req.addArgumentByValue("param",params);
	    req.addArgumentByValue("resolve",data);
	    INKFResponseReadOnly page=context.issueRequestForResponse(req);
	    INKFResponse resp=context.createResponseFrom( page );
	    resp.setHeader("WrappedControlPanel", true);
	}
}