<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml"/>
	<xsl:param name="config"/>
	<xsl:param name="uptime"/>
	
	<xsl:template match="/*">
		<table class="data-table">
			<tr class="vis-title">
				<td class="vis-title" colspan="2">Host Operating System</td>
			</tr>
			<tr>	
				<td class="label">Name:</td>
				<td><xsl:value-of select="system-properties/property[name='os.name']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Architecture:</td>
				<td><xsl:value-of select="system-properties/property[name='os.arch']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Version:</td>
				<td><xsl:value-of select="system-properties/property[name='os.version']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Hostname:</td>
				<td><xsl:value-of select="hostname"/> (<xsl:value-of select="ip"/>)</td>
			</tr>
			<xsl:if test="physical">
				<tr>	
					<td class="label">Physical Mem:</td>
					<td><xsl:value-of select="format-number(physical div (1024*1024), '#')"/>Mb
					</td>
				</tr>
			</xsl:if>
			<tr>	
				<td class="label">Uptime:</td>
				<td><xsl:value-of select="osuptime"/></td>
			</tr>
			
			<tr class="vis-title">
				<td colspan="2">Java Virtual Machine</td>
			</tr>
			<tr>	
				<td class="label">Name:</td>
				<td><xsl:value-of select="system-properties/property[name='java.vm.name']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Version:</td>
				<td><xsl:value-of select="system-properties/property[name='java.version']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Vendor:</td>
				<td><xsl:value-of select="system-properties/property[name='java.vm.vendor']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Info:</td>
				<td><xsl:value-of select="system-properties/property[name='java.vm.info']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Process Id:</td>
				<td><xsl:value-of select="pid"/></td>
			</tr>
			<tr>	
				<td class="label">Uptime:</td>
				<td><xsl:value-of select="$uptime/uptime/displayString"/></td>
			</tr>
			<tr>	
				<td class="label">CPU Cores:</td>
				<td><xsl:value-of select="processors"/></td>
			</tr>
			<tr>
				<td class="label">Memory Pools:</td>
				<td>
					<xsl:apply-templates select="memory-pools/pool"/>
				</td>
			</tr>
			<tr>
				<td class="label">Garbage Collector:</td>
				<td>
					<xsl:for-each select="garbage-collectors/collector">
						<div><xsl:value-of select="name"/> (<xsl:value-of select="count"/> collections in <xsl:value-of select="time"/>ms)
						</div>
					</xsl:for-each>
				</td>
			</tr>
			<xsl:if test="resident">
				<tr>	
					<td class="label">Resident Mem:</td>
					<td><xsl:value-of select="format-number(resident div (1024*1024), '#.0')"/>Mb
					</td>
				</tr>
				<tr>	
					<td class="label">Virtual Mem:</td>
					<td><xsl:value-of select="format-number(virtual div (1024*1024), '#.0')"/>Mb
					</td>
				</tr>
			</xsl:if>
			<tr>	
				<td class="label">Headless AWT:</td>
				<td>
					<xsl:variable name="headless" select="system-properties/property[name='java.awt.headless']/value"/>
					<xsl:choose>
						<xsl:when test="$headless"><xsl:value-of select="headless"/></xsl:when>
						<xsl:otherwise>false</xsl:otherwise>
					</xsl:choose>
				</td>
			</tr>
			<tr>	
				<td class="label">Encoding:</td>
				<td><xsl:value-of select="system-properties/property[name='file.encoding']/value"/></td>
			</tr>


			<tr class="vis-title">
				<td colspan="2">NetKernel</td>
			</tr>
			<tr>	
				<td class="label">Product:</td>
				<td><xsl:value-of select="$config/config/item[name='netkernel.instance.product']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Major Version:</td>
				<td><xsl:value-of select="$config/config/item[name='netkernel.instance.version.major']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Major Minor:</td>
				<td><xsl:value-of select="$config/config/item[name='netkernel.instance.version.minor']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Instance Id:</td>
				<td><xsl:value-of select="$config/config/item[name='netkernel.instance.identifier']/value"/></td>
			</tr>
			<tr>	
				<td class="label">Total Requests:</td>
				<td><xsl:value-of select="totalRequests"/></td>
			</tr>
			<tr>	
				<td class="label">Recent Requests:</td>
				<td><xsl:value-of select="requestsInPeriod"/> in last <xsl:value-of select="round(statPeriod div 60)"/> minutes (<xsl:value-of select="round(requestRateInPeriod * 60)"/> req/min)</td>
			</tr>
			
			<tr>	
				<td class="label">Configuration:</td>
				<td>
					<table class="data-table">
						<tr class="vis-title">
							<td>Name</td><td>Value</td>
						</tr>
						<xsl:for-each select="$config/config/item">
								<tr>
									<td><xsl:value-of select="name"/></td>
									<td><xsl:value-of select="value"/></td>
								</tr>
						</xsl:for-each>
						</table>
				</td>
			</tr>			
			
			<tr>	
				<td class="label">Boot Classpath:</td>
				<td>
					<xsl:for-each select="boot-classpath/element">
						<div class="lowrow">
							<xsl:choose>
								<xsl:when test="title">
									<xsl:value-of select="title"/> (v<xsl:value-of select="version"/>)
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="url"/>
								</xsl:otherwise>
							</xsl:choose>
						</div>
					</xsl:for-each>
				</td>
			</tr>
			<tr>	
				<td class="label">Runlevel:</td>
				<td><xsl:value-of select="runlevel"/></td>
			</tr>
			<tr>	
				<td class="label">Modules:</td>
				<td>
					<xsl:for-each select="modules/module">
						<xsl:sort select="runlevel" data-type="number"/>
						<xsl:sort select="identifier"/>
						<div class="lowrow">
							<xsl:value-of select="runlevel"/><xsl:text> </xsl:text><xsl:value-of select="identifier"/> (v<xsl:value-of select="version"/>)
						</div>
					</xsl:for-each>
				</td>
			</tr>
		</table>
	</xsl:template>
	
	<xsl:template match="pool">
		<div style="margin-top: 10px;" class="lowrow">
			<xsl:variable name="percent" select="used div max"/>
						
			<div style="position:absolute; z-index:1;"><xsl:value-of select="format-number(used div (1024*1024), '0.0')"/>Mb</div>
			
   			<span title="" style="border: solid 1px #444; text-align: left; ">
				<span style="background: #7a72c8; padding-left: {$percent*75}px;"><img src="/nkse/style/img/transparent.gif" width="0" height="8"/></span>
				<span style="background: white; padding-left: {(1-$percent)*75}px;"><img src="/nkse/style/img/transparent.gif" width="0" height="8"/></span>		
			</span>
			<xsl:text> </xsl:text>
			<xsl:value-of select="name"/> 
			(Maximum size: <xsl:value-of select="format-number(max div (1024*1024), '0.0')"/>Mb)
		</div>
	</xsl:template>

</xsl:stylesheet>