<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xrl="http://netkernel.org/xrl">
	<xsl:output method="xml"/>
	
	<xsl:template match="/k">
		<div>
		<xrl:include>
			<xrl:identifier>active:controlPanelInfoBox</xrl:identifier>
			<xrl:argument name="content">
				<div>
				<p>
          The Requests and Threads tool provides a view of the internal state of the NetKernel microkernel.
        </p>
				<p>
          It shows the currently active root requests and how they translate onto the physical execution threads.
        </p>
				<p>
          The <b>Root Requests</b> section lists all active transport initiated requests. The status
				column shows a diagnostic execution status which may be one of  OK, QUEUED, BUSY_ENDPOINT,
				RUNAWAY, DEADLOCKED, RESPONSELESS, LONG_WAIT, LONG_IO_WAIT or THREAD_STARVED.
          Clicking on the button labeled &quot;kill&quot; next to a root request will
          cause NetKernel to make its best attempt to terminate execution of that request.
          (Timings are expressed in milliseconds)
        </p>
				<p>
          The <b>Kernel Worker Threads</b> section shows all kernel and transport threads
          available to service requests.
          Clicking on a thread to see a detailed stack trace and details of any held locks.
          Threads may be in the states of  RUNNABLE, BLOCKED, WAITING, TIMED_WAITING, NEW or TERMINATED.
          See
				  <a href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Thread.State.html">Thread.State</a>
          for more information.
        </p>
				<p>
          The <b>Locked Endpoints</b> section shows any non-threadsafe endpoints which are handling requests
          along with requests they are blocking.
        </p>
				<p>
          The <b>Request Queue</b> section shows requests waiting a worker thread.
          This queue is typically empty unless a system uses a large number of asynchronous requests.
        </p>
				<p>
         The <b>Other JVM Threads</b> section lists all the other threads within the Java virtual machine
          along with their status.
        </p>
				</div>
			</xrl:argument>
		</xrl:include>
		
		<h2>Root Requests</h2>
		<table class="data-table">
		<tr class="vis-title">
			<td>Root Id</td><td>Request</td><td>Duration</td><td>Last Activity</td><td>Status</td><td/>
		</tr>
		<xsl:for-each select="roots/state">
			<xsl:sort select="root-id" data-type="number"/>
			<xsl:variable name="class">
				<xsl:choose>
					<xsl:when test="position() mod 2 = 0">
						<xsl:text>evenrow</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>oddrow</xsl:text>
					</xsl:otherwise>  
				</xsl:choose>
			</xsl:variable>
			<tr class="{$class}">	
				<xsl:if test="status!='OK'">
					<xsl:attribute name="class">error</xsl:attribute>
				</xsl:if>
				<td><xsl:value-of select="root-id"/></td>
				<td><xsl:value-of select="request"/>
				</td>
				<td><xsl:value-of select="duration"/></td>
				<td><xsl:value-of select="active"/></td>
				<td><xsl:value-of select="status"/></td>
				<td>
					<input type="button" value="kill" onclick="parent.location='/tools/kill/{root-id}'" class="btn"/>
				</td>
			</tr>
		</xsl:for-each>
		</table>
		
		
		<h2>Kernel Worker Threads</h2>
		<table class="data-table">
		<tr class="vis-title">
			<td>Thread</td><td>Status</td><td>CPU</td><td>Root</td><td>Last Activity</td>
		</tr>
		<xsl:for-each select="threads/thread[isKernel='true']">
			<xsl:sort select="name"/>
			<xsl:variable name="class">
				<xsl:choose>
					<xsl:when test="position() mod 2 = 0">
						<xsl:text>evenrow</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>oddrow</xsl:text>
					</xsl:otherwise>  
				</xsl:choose>
			</xsl:variable>
			<tr class="{$class}">	
				<td><a href="#{name}"><xsl:value-of select="name"/></a></td>
				<td><xsl:value-of select="state"/></td>
				<td><xsl:value-of select="cpu"/></td>
				<td><xsl:value-of select="root"/></td>
				<td><xsl:value-of select="request"/>
				</td>
			</tr>
		</xsl:for-each>
		</table>

		<h2>Locked Endpoints</h2>
		<table class="data-table">
		<tr class="vis-title">
			<td>Endpoint/Request</td><td>Root</td><td>Queued Duration</td>
		</tr>
		<xsl:for-each select="locked-endpoints/endpoint">
			<xsl:variable name="class">
				<xsl:choose>
					<xsl:when test="position() mod 2 = 0">
						<xsl:text>evenrow</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>oddrow</xsl:text>
					</xsl:otherwise>  
				</xsl:choose>
			</xsl:variable>
			<tr class="{$class}">	
				<td><xsl:value-of select="class"/></td>
			</tr>
			<xsl:for-each select="queued/state">
				<tr class="{$class}">
					<td style="padding-left: 1em;"><xsl:value-of select="request"/></td>
					<td><xsl:value-of select="root-id"/></td>
					<td><xsl:value-of select="active"/></td>
				</tr>
			</xsl:for-each>
		</xsl:for-each>
		</table>

		<h2>Request Queue</h2>
		<table class="data-table">
		<tr class="vis-title">
			<td>Priority</td><td>Request</td><td>Root</td><td>State</td><td>Queued Duration</td>
		</tr>
		<xsl:for-each select="queue/state">
			<xsl:variable name="class">
				<xsl:choose>
					<xsl:when test="position() mod 2 = 0">
						<xsl:text>evenrow</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>oddrow</xsl:text>
					</xsl:otherwise>  
				</xsl:choose>
			</xsl:variable>
			<tr class="{$class}">	
				<td><xsl:value-of select="priority"/></td>
				<td><xsl:value-of select="request"/></td>
				<td><xsl:value-of select="root-id"/></td>
				<td><xsl:value-of select="state"/></td>
				<td><xsl:value-of select="active"/></td>
			</tr>
		</xsl:for-each>
		</table>

		<h2>Other JVM Threads</h2>
		<table class="data-table">
		<tr class="vis-title">
			<td>Thread</td><td>Status</td><td>CPU</td>
		</tr>
		<xsl:for-each select="threads/thread[isKernel!='true']">
			<xsl:sort select="name"/>
			<xsl:variable name="class">
				<xsl:choose>
					<xsl:when test="position() mod 2 = 0">
						<xsl:text>evenrow</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>oddrow</xsl:text>
					</xsl:otherwise>  
				</xsl:choose>
			</xsl:variable>
			<tr class="{$class}">	
				<td><a href="#{name}"><xsl:value-of select="name"/></a></td>
				<td><xsl:value-of select="state"/></td>
				<td><xsl:value-of select="cpu"/></td>
			</tr>
		</xsl:for-each>
		</table>

		<h2>Thread Detail</h2>
		<table class="data-table">
		<tr class="vis-title">
			<td>Thread</td><td>Status</td><td>Root</td><td>Last Activity</td>
		</tr>
		<xsl:for-each select="threads/thread">
			<xsl:sort select="name"/>
			<xsl:variable name="class">
				<xsl:choose>
					<xsl:when test="position() mod 2 = 0">
						<xsl:text>evenrow</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>oddrow</xsl:text>
					</xsl:otherwise>  
				</xsl:choose>
			</xsl:variable>
			<tr class="{$class}">	
				<td><a name="{name}"/><xsl:value-of select="name"/></td>
				<td><xsl:value-of select="state"/></td>
				<td><xsl:value-of select="root"/></td>
				<td><xsl:value-of select="request"/></td>
			</tr>
			<xsl:if test="lock">
				<tr class="{$class}"><td/>
					<td colspan="4">Lock name: [<xsl:value-of select="lock"/>] Lock owner: [<xsl:value-of select="lock-owner"/>]
					</td>
				</tr>
			</xsl:if>
			
			<tr class="{$class}"><td/>
				<td colspan="4">
					<xsl:for-each select="stack/frame">
						<div style="font-size: 66%;"><xsl:value-of select="."/></div>
					</xsl:for-each>
				</td></tr>
		</xsl:for-each>
		</table>
		</div>
	</xsl:template>
	
</xsl:stylesheet>
